/*
 * TCS Dependency Map Renderer
 *
 * Copyright (c) 2005 TopCoder, Inc. All rights reserved.
 *
 * Author: Pops, TCSDeveloper
 */

using System;
using System.Collections;
using System.IO;
using NUnit.Framework;

namespace TopCoder.Rules.DependencyMap.View.StressTests
{
    /**
     * Stress tests Task 4
     */
    [TestFixture]
    public class StressBlankColumnRemoval 
    {
        /**
        * Dependency purposely built during setup to have
        * a large number of sub-elements
        */
        Dependency myDependency;

        /**
        * number of levels the test dependency tree will have
        */
        private const int maxNumLevels = 6;

        /**
        * number of sub-elements for each level of the test dependency
         * this number is even so as to avoid the blank column issue
        */
        private const int maxNumElements = 4;

        /**
        * List of Diagram renderers
        */
        private IDiagramRenderer sampleRenderer;

        /**
        * Template name for the test files
        */
        private const string fileName = "../../test_files/file00";

        /**
        * counter for number of dependants
        */
        private int totalCount;

        [SetUp]
        public void setUp()
        {
            sampleRenderer = new SVGRenderer(fileName);
            //create the large dependency graph
            totalCount = 1;
            myDependency = new Dependency("evenChildrenDependency");
            Queue currentLevel = new Queue();
            currentLevel.Enqueue(myDependency);
            for(int i = 0; i < maxNumLevels; ++i)
            {
                Queue nextLevel = new Queue();
                while(currentLevel.Count > 0)
                {
                    Dependency current = (Dependency)currentLevel.Dequeue();
                    for(int j = 0; j < maxNumElements; ++j)
                    {
                        Dependency d = new Dependency("level " + i + " element " + j);
                        current.AddDependant(d);
                        nextLevel.Enqueue(d);
                        ++totalCount;
                    }
                }
                currentLevel = new Queue(nextLevel);
            }
        }

        /**
        * Removes the temporary files
        */
        [TearDown]
        public void TearDown()
        {
            File.Delete(fileName);
        }

        /**
        * Attempts to render a large dependency graph
        */
        [Test]
        public void TestEvenDescendants()
        {
            // Create the Dependency Renderer and add a diagram renderer
            DependencyRenderer renderer = new DependencyRenderer();
            renderer.AddDiagramRenderer(sampleRenderer);
            DateTime startTime = DateTime.Now;
            // Render the dependency
            renderer.Render(myDependency);
            TimeSpan ts = DateTime.Now - startTime;
            Console.WriteLine("Rendering a graph with even descendants " + maxNumLevels + " levels and " + maxNumElements + " elements per level  (total of " + totalCount + " elements) took " + ts.TotalMilliseconds + " ms");
        }
}
}