/*
 * TCS Dependency Map Renderer
 *
 * Copyright (c) 2005 TopCoder, Inc. All rights reserved.
 *
 * Author: Pops, TCSDeveloper
 */

using System;
using System.IO;
using NUnit.Framework;

namespace TopCoder.Rules.DependencyMap.View.StressTests
{
    /**
     * Stress tests Task 5
     */
    [TestFixture]
    public class StressDuplicateRemoval 
    {
        /**
        * Dependency purposely built during setup to have
        * a large number of common dependencies at a low level
        */
        Dependency lotsOfSwapsDependency;

        /**
        * number of dependencies of the main element
        */
        private const int maxNumChildren = 100;

        /**
        * number of common sub-dependencies for the direct dependencies
        */
        private const int maxNumCommon = 4;

        /**
        * List of Diagram renderers
        */
        private IDiagramRenderer sampleRenderer;

        /**
        * Template name for the test files
        */
        private const string fileName = "../../test_files/file00";

        /**
         *
         */
        [SetUp]
        public void setUp()
        {
            sampleRenderer = new SVGRenderer(fileName);
            //create the sample dependency graphs
            lotsOfSwapsDependency = new Dependency("some stuff");
            //creates the common dependents array and fill it
            Dependency[] commons = new Dependency[maxNumCommon];
            for(int i = 0; i < maxNumCommon; ++i)
            {
                commons[i] = new Dependency("common" + i);
            }

            for(int i = 0; i < maxNumChildren; ++i)
            {
                Dependency child = new Dependency("child" + i);
                for(int j = 0; j < maxNumCommon; ++j)
                {
                    child.AddDependant(commons[j]);
                }
                lotsOfSwapsDependency.AddDependant(child);
            }
        }

        /**
        * Removes the temporary files
        */
        [TearDown]
        public void TearDown()
        {
            File.Delete(fileName);
        }

        /**
        * Attempts to render a large dependency graph
        */
        [Test]
        public void TestManySwaps()
        {
            // Create the Dependency Renderer and add a diagram renderer
            DependencyRenderer renderer = new DependencyRenderer();
            renderer.AddDiagramRenderer(sampleRenderer);
            DateTime startTime = DateTime.Now;
            // Render the dependency
            renderer.Render(lotsOfSwapsDependency);
            TimeSpan ts = DateTime.Now - startTime;
            Console.WriteLine("Rendering a graph with " + maxNumChildren + " elements and " + maxNumCommon + " common dependencies per element took " + ts.TotalMilliseconds + " ms");
        }
    }
}